@@ -4,9 +4,10 @@ import pickle
 import sys
 import traceback
 import warnings
+import datetime
 
 
-def mute_warnings():
+def mute_warnings():  # pragma: no cover
     """
     Mute all Python warnings
     """
@@ -14,12 +15,12 @@ def mute_warnings():
     warnings.filterwarnings("ignore")
 
 
-def screen_width_full():
+def screen_width_full():  # pragma: no cover
     from IPython.core.display import display, HTML
     display(HTML("<style>.container { width:100% !important; }</style>"))
 
 
-def print_error(tag='', verbose=False, buffer=None):
+def print_error(tag='', verbose=False, buffer=None):  # pragma: no cover
     """
     Function for printing errors in except block.
     Args:
@@ -41,18 +42,6 @@ def print_error(tag='', verbose=False, buffer=None):
     print(f'{exc_type.__name__}: {exc_value}{tag}', file=buffer)
 
 
-class ChattyDict(dict):
-    """
-    A dict subclass that throws keyerror with existing
-    keys.
-    """
-    def __getitem__(self, key):
-        try:
-            return super().__getitem__(key)
-        except KeyError:
-            raise KeyError(f'{key!r} not in {list(self.keys())}')
-
-
 class cached_property(object):
     """
     Decorator that converts a method with a single self argument into a
@@ -108,7 +97,9 @@ class cached_container(object):
         if instance is None:
             return self
 
-        cached_var_name = '_cached_frame_for_' + self.func.__name__
+        cached_var_name = '_cached_container_for_' + self.func.__name__
+        self._cached_var_name = cached_var_name
+        
         if cached_var_name not in instance.__dict__:
             instance.__dict__[cached_var_name] = self.func(instance)
         try:
@@ -116,9 +107,12 @@ class cached_container(object):
         except AttributeError:
             out = copy.copy(instance.__dict__[cached_var_name])
         return out
+    
+    def __delete__(self, obj):
+        delattr(obj, self._cached_var_name)
 
 
-class cached_dataframe(cached_container):
+class cached_dataframe(cached_container):  # pragma: no cover
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         warnings.warn('@cached_dataframe is deprecated.  @Use cached_container')
@@ -129,15 +123,13 @@ class pickle_cache_state:
     This is a descriptor that stores optional state for pickle cache
     """
     def __init__(self, mode=None):
-        allowed_modes = ['active', 'ignore', 'purge', 'refresh']
+        allowed_modes = ['active', 'ignore', 'refresh', 'reset', 'memory']
         if mode not in allowed_modes:
             raise ValueError(f'You must set mode to be one of {allowed_modes}')
         self.mode = mode
 
     def __get__(self, instance, owner):
-        for att in owner.__dict__.values():
-            if isinstance(att, pickle_cache_state):
-                print('pickle_mute mode', att.mode)
+        pass
 
     def __set__(self, instance, value):
         raise NotImplementedError('You cannot set this attribute')
@@ -168,8 +160,9 @@ class pickle_cached_container:
         ## Modes:
         ##   'active': same as not even specifying this class attribute
         ##   'ignore': maintains all pickle files as they were but ignores the cache
-        ##   'purge': deletes corresponding pickle file and ignores the cache
-        ##   'refresh': Reccomputes and refreshes pickle file
+        ##   'refresh': Recomputes and refreshes pickle file
+        ##   'reset': Alias for refresh
+        ##   'memory': Will recompute and cache on object only.  Not to file.
         pcs = ezr.pickle_cache_state(mode='active')
 
         # Default cache file /tmp/<cls>.<meth>.pickle
@@ -215,12 +208,14 @@ class pickle_cached_container:
         decorator/descriptor class.
         """
         self.func = func
+        self.cached_var_name = '_pickle_cache_for_' + self.func.__name__
         return self
 
     @property
     def default_pickle_file_name(self):
-        return '/tmp/{}.pickle'.format(
-            self.func.__qualname__.lower()
+        return '/tmp/{}_{}.pickle'.format(
+            self.func.__qualname__,
+            str(datetime.datetime.now().date())
         )
 
     @property
@@ -230,62 +225,21 @@ class pickle_cached_container:
         else:
             return self.default_pickle_file_name
 
-    def __get__(self, instance, type=None):
-        """
-        After decoration the decorated method will be replaced with
-        an instance of the decorator/descriptor.  Every time this instance
-        is accessed this method will be called to return the value
-        of the method, which has been turned into a pickle-backed property.
-        """
-        # This is extra logic that will check for pickle cache state
+    def _get_cache_mode(self, instance):
+        cache_mode = None
         for att in instance.__class__.__dict__.values():
             # If a cache state attribute was found on the class
             if isinstance(att, pickle_cache_state):
                 # Get the mode from the state
                 cache_mode = att.mode
+                break
+        return cache_mode
 
-                # If cache mode is refresh, then delete and recache
-                if cache_mode == 'refresh':
-                    self.__delete__(instance)
-
-                # If cache mode is purge, then just delete and run callable
-                elif cache_mode == 'purge':
-                    self.__delete__(instance)
-                    return self.func(instance)
-
-                # If ignoring the cache, don't do anything to file, just return callable
-                elif cache_mode == 'ignore':
-                    return self.func(instance)
-
-        # # This was part of the original django cached_property code.
-        # # I don't think I need it. I'm going to leave it in here commented
-        # # out though for reference.
-        # if instance is None:
-        #     return self
-
-        # An attribute of this name will be placed on the object having
-        # the decorated method
-        cached_var_name = '_pickle_cache_for_' + self.func.__name__
-
-        # Populate the in-memory cache if needed
-        if cached_var_name not in instance.__dict__:
-            instance.__dict__[cached_var_name] = self.get_pickled(instance)
-
-        # If you don't need to return a copy, just return the cached object itself
-        if not self.return_copy:
-            return instance.__dict__[cached_var_name]
-
-        # Otherwise return a copy of the cached object
-        try:
-            out = instance.__dict__[cached_var_name].copy()
-        except AttributeError:
-            out = copy.copy(instance.__dict__[cached_var_name])
-        return out
-
-    def get_pickled(self, instance):
+    def _get_pickle_or_compute(self, instance):
         """
         This method either pulls data from pickle file if it exists
-        otherwise it will populate the pickle file.
+        otherwise it will compute results, populate the pickle file and
+        return the computed object.
         """
         # If pickle file exists, load its contents
         if os.path.isfile(self.pickle_file_name):
@@ -300,14 +254,86 @@ class pickle_cached_container:
 
         return obj
 
+    def _get_memory_pickle_or_compute(self, instance):
+        """
+        Try to get the object trying in this order:
+        1) From memory cached on the object
+        2) From the pickle file
+        3) By running the decorated method
+
+        The retrieved value will be stored on the object if it isn't
+        already there
+        """
+        if self.cached_var_name not in instance.__dict__:
+            instance.__dict__[self.cached_var_name] = self._get_pickle_or_compute(instance)
+        return instance.__dict__[self.cached_var_name]
+
+    def _get_memory_or_compute(self, instance):
+        """
+        Try to get the object trying in this order:
+        1) From memory cached on the object
+        2) By running the decorated method
+
+        The retrieved value will be stored on the object if it isn't
+        already there
+        """
+        if self.cached_var_name not in instance.__dict__:
+            instance.__dict__[self.cached_var_name] = self.func(instance)
+        return instance.__dict__[self.cached_var_name]
+
+    def _get_or_compute(self, instance, cache_mode):
+        # If ignoring the cache, always call the decorated method
+        if cache_mode == 'ignore':
+            return self.func(instance)
+        # If memory, ignore the pickle file but use object caching
+        elif cache_mode == 'memory':
+            return self._get_memory_or_compute(instance)
+
+        # This looks weird, but it is the same thing as deleting
+        # the pickle-cached property on the host object.  Doing so
+        # will bust the cache. So refreshing busts the cache
+        # and repopulates it by computing.
+        elif cache_mode in ['refresh', 'reset']:
+            self.__delete__(instance)
+            return self._get_memory_pickle_or_compute(instance)
+
+        # Otherwise use object and pickle caching
+        else:
+            return self._get_memory_pickle_or_compute(instance)
+
+    def _copy_object(self, obj):
+        try:
+            out = obj.copy()
+        except AttributeError:
+            out = copy.copy(obj)
+        return out
+
+    def __get__(self, instance, type=None):
+        """
+        After decoration the decorated method will be replaced with
+        an instance of the decorator/descriptor.  Every time this instance
+        is accessed this method will be called to return the value
+        of the method, which has been turned into a pickle-backed property.
+        """
+        # Get the cache mode
+        cache_mode = self._get_cache_mode(instance)
+
+        # Grab the object, (persisting to cache if required)
+        obj = self._get_or_compute(instance, cache_mode)
+
+        # Copy the object if appropriate
+        if self.return_copy:
+            return self._copy_object(obj)
+        else:
+            return obj
+
     def __delete__(self, instance):
         """
         This method handles busting the cache.
         """
         # Delete the cached copy of the data
-        cached_var_name = '_pickle_cache_for_' + self.func.__name__
-        if cached_var_name in instance.__dict__:
-            del instance.__dict__[cached_var_name]
+        if self.cached_var_name in instance.__dict__:
+            del instance.__dict__[self.cached_var_name]
 
         # Delete the pickle file
         if os.path.isfile(self.pickle_file_name):
